import numpy as np

def MinMaxScaler(X, feature_range=(0, 1)):
    X = np.array(X)
    min_range, max_range = feature_range  

    X_min = X.min(axis=0)
    X_max = X.max(axis=0)

    denom = np.where(X_max - X_min == 0, 1, X_max - X_min)

    X_scaled = (X - X_min) / denom * (max_range - min_range) + min_range
    return X_scaled



def TrainTestSplit(X, Y, test_size=0.2):
    X_test = X[:int(len(X) * test_size)]
    X_train = X[int(len(X) * test_size):]
    Y_test = Y[:int(len(Y) * test_size)]
    Y_train = Y[int(len(Y) * test_size):]
    return X_train, X_test, Y_train, Y_test


import pandas as pd
def HandleMissingValue(X):
    # Calculate missing percentage per column
    null_percentage = X.isnull().sum() / len(X) * 100

    # Split columns by missing percentage
    null_percentage_most = null_percentage[null_percentage > 40].index.tolist()
    null_percentage_less = null_percentage[null_percentage < 40].index.tolist()

    # Drop high-missing columns
    X.drop(columns=null_percentage_most, inplace=True)

    # Fill low-missing categorical columns with mode
    for col in null_percentage_less:
        if X[col].dtype == "object" or X[col].dtype.name == "category":
            X[col] = X[col].fillna(X[col].mode()[0])
        else:
            X[col] = X[col].fillna(X[col].median())  # better for numeric


    categorical_columns = X.select_dtypes(include=["object", "category"]).columns.tolist()
    X_encoded = pd.get_dummies(X, columns=categorical_columns)
    return X_encoded


import numpy as np
class LinearRegression():
    def __init__(self, x_train, y_train, x_test, y_test, lr=.03, epochs=500):
        self.w = np.zeros((x_train.shape[1], 1)).astype(float)
        self.b = 0.0
        self.lr = lr
        self.epochs = epochs
        self.x_train = x_train.values.astype(float)
        self.y_train = y_train.values.reshape(-1, 1).astype(float)

    def predict(self, X, w, b):
        y = X @ w + b
        return y

    def regression(self):
        m = self.x_train.shape[0]
        for i in range(self.epochs):
            y_pred = self.predict(self.x_train, self.w, self.b)

            # loss
            mse = 1 / (2 * m) * ((y_pred - self.y_train).T @ (y_pred - self.y_train))

            # print(f"Mean Square Error is: {mse} in epoch: {i}")

            # gradiant
            dw = 1 / m * (self.x_train.T @ (y_pred - self.y_train))
            db = 1 / m * np.sum(y_pred - self.y_train)

            # update weights
            self.w -= self.lr * dw
            self.b -= self.lr * db
        return self.w, self.b



from .LinearRegression import LinearRegression
from .HandleMissingValue import HandleMissingValue
from .TrainTestSplit import TrainTestSplit
from .MinMaxScaler import MinMaxScaler  

__all__ = ["LinearRegression", "HandleMissingValue", "TrainTestSplit", "MinMaxScaler"]
